/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2006 Pierre Habouzit
 */

#ifndef MUTT_LIB_LIB_STR_H
#define MUTT_LIB_LIB_STR_H

#include <string.h>
#include "../lib/str.h"

#include "mem.h"

#define NONULL(x) (x?x:"")

static inline int m_strisempty(const char *s) {
    return !s || !*s;
}

static inline ssize_t m_strlen(const char *s) {
    return s ? strlen(s) : 0;
}

static inline ssize_t m_strnlen(const char *s, ssize_t n) {
    const char *p = memchr(s, '\0', n);
    return p ? p - s : n;
}

static inline char *m_strdup(const char *s) {
    ssize_t len = m_strlen(s);
    return len ? p_dup(s, len + 1) : NULL;
}

static inline int m_strcmp(const char *a, const char *b) {
    return strcmp(NONULL(a), NONULL(b));
}

static inline int m_strcasecmp(const char *a, const char *b) {
    return strcasecmp(NONULL(a), NONULL(b));
}

static inline int m_strncmp(const char *a, const char *b, size_t n) {
    return strncmp (NONULL(a), NONULL(b), n);
}

static inline int m_strncasecmp(const char *a, const char *b, size_t n) {
    return strncasecmp(NONULL(a), NONULL(b), n);
}


ssize_t m_strcpy(char *dst, ssize_t n, const char *src);
ssize_t m_strncpy(char *dst, ssize_t n, const char *src, ssize_t l);

static inline ssize_t m_strcat(char *dst, ssize_t n, const char *src) {
    ssize_t dlen = m_strnlen(dst, n - 1);
    return dlen + m_strcpy(dst + dlen, n - dlen, src);
}

static inline ssize_t
m_strncat(char *dst, ssize_t n, const char *src, ssize_t l) {
    ssize_t dlen = m_strnlen(dst, n - 1);
    return dlen + m_strncpy(dst + dlen, n - dlen, src, l);
}

#endif /* MUTT_LIB_LIB_STR_H */
